import { USERNAME_REPOSITORY } from '@modules/authentication/authentication.di-tokens';
import { DeleteUsernameCommand } from '@modules/authentication/core/application/commands/delete-username/delete-username.command';
import { DeleteUsernameService } from '@modules/authentication/core/application/commands/delete-username/delete-username.service';
import { DeleteUsernameRequestDto } from '@modules/authentication/interface/grpc-controllers/dtos/delete-username.request.dto';
import { UnauthorizedException } from '@nestjs/common';
import { Test, TestingModule } from '@nestjs/testing';

const deleteUsernameRequest: DeleteUsernameRequestDto = {
  name: 'john.doe@email.com',
};

const mockUsernameEntity = {
  delete: jest.fn(),
  getProps: jest.fn().mockImplementation(() => ({
    userId: 'b1643072-757f-4e3b-87f2-63af3684afb8',
  })),
};

const mockUsernameRepository = {
  findByName: jest.fn().mockImplementation(() => mockUsernameEntity),
  countUsernames: jest
    .fn()
    .mockImplementationOnce(() => 2)
    .mockImplementationOnce(() => 1),
  deleteUsername: jest.fn().mockImplementationOnce(() => true),
};

describe('Delete Username Service', () => {
  let deleteUsernameService: DeleteUsernameService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USERNAME_REPOSITORY,
          useValue: mockUsernameRepository,
        },
        DeleteUsernameService,
      ],
    }).compile();

    deleteUsernameService = module.get<DeleteUsernameService>(
      DeleteUsernameService,
    );
  });

  it('should be defined', () => {
    expect(deleteUsernameService).toBeDefined();
  });

  describe('execution', () => {
    const deleteUsernameCommand = new DeleteUsernameCommand(
      deleteUsernameRequest,
    );
    it('should delete a username', async () => {
      const result: boolean = await deleteUsernameService.execute(
        deleteUsernameCommand,
      );
      expect(result).toBeTruthy();
    });
    it('should throw an exception when trying to delete the last username', async () => {
      await expect(
        deleteUsernameService.execute(deleteUsernameCommand),
      ).rejects.toBeInstanceOf(UnauthorizedException);
    });
  });
});
