import { Type } from '@modules/authentication/core/domain/username.types';
import { IsValidUsername } from '@modules/authentication/interface/grpc-controllers/dtos/validators/decorators/is-valid-username.decorator';
import { Validator } from 'class-validator';

describe('Username decorator', () => {
  class MyClass {
    @IsValidUsername({
      message: 'Invalid username',
    })
    name: string;

    type: Type;
  }
  it('should return a property decorator has a function', () => {
    const isValidUsername = IsValidUsername();
    expect(typeof isValidUsername).toBe('function');
  });
  it('should validate a valid phone username', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.name = '+33611223344';
    myClassInstance.type = Type.PHONE;
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(0);
  });
  it('should validate a valid email username', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.name = 'john.doe@email.com';
    myClassInstance.type = Type.EMAIL;
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(0);
  });
  it('should not validate an invalid phone username', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.name = '11223344';
    myClassInstance.type = Type.PHONE;
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(1);
  });
  it('should not validate an invalid email username', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.name = 'john.doe.email.com';
    myClassInstance.type = Type.EMAIL;
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(1);
  });
  it('should not validate if type is not set', async () => {
    const myClassInstance = new MyClass();
    myClassInstance.name = 'john.doe@email.com';
    const validator = new Validator();
    const validation = await validator.validate(myClassInstance);
    expect(validation.length).toBe(1);
  });
});
