import { RpcExceptionCode, RpcValidationPipe } from '@mobicoop/ddd-library';
import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DecisionRequestDto } from './dtos/decision.request.dto';
import { DecisionQuery } from '@modules/authorization/core/application/queries/decision/decision.query';
import { DecisionResponseDto } from '../dtos/decision.response.dto';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class DecideGrpcController {
  constructor(private readonly queryBus: QueryBus) {}

  @GrpcMethod('AuthorizationService', 'Decide')
  async decide(data: DecisionRequestDto): Promise<DecisionResponseDto> {
    try {
      const allow: boolean = await this.queryBus.execute(
        new DecisionQuery(data.domain, data.action, data.context),
      );
      return new DecisionResponseDto(allow);
    } catch (error: any) {
      throw new RpcException({
        code: RpcExceptionCode.PERMISSION_DENIED,
        message: 'Permission denied',
      });
    }
  }
}
